/* $Workfile: OsaLinuxRtl.h $   $Revision: 29 $    $Modtime: 5/23/08 11:27a $  */
/*******************************************************************************

  Odin TeleSystems Inc.

  OTX Telecom Adapter Driver

  @doc internal

  @module

  @normal Copyright (C) Odin TeleSystems Inc., 1998-2003, All Rights Reserved

*******************************************************************************/

#ifndef OSA_LINUX_RTL_H
#define OSA_LINUX_RTL_H

#include <linux/version.h>  //KERNEL_VERSION(2,4,0)

// The following defines and inludes ensures the driver uses
// versioned kernel symbols to avoid problems with the driver and
// kernel being incompatible due to kernel API changes
#include <linux/autoconf.h>
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0)
#if defined(CONFIG_MODVERSIONS) && !defined(MODVERSIONS)
# define MODVERSIONS // Assure we are using symbol versioning
#endif 

#ifdef MODVERSIONS
# include <linux/modversions.h> // Version the symbols to be included
#endif
#endif


#include "../OsaDef.h"


#if DBG

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)
// Setting this value to 1 will install a stack check into the OSA_DPRINTF
// that will dump a call stack track if the stack is less then 1000 bytes.
#define STACK_CHECK_ENABLED 1
#endif


#if STACK_CHECK_ENABLED == 1

#ifdef __cplusplus
extern "C" {
#endif

extern OsaInt32 OsaGetKernelStackAvailable(OsaUint32 *stackAvailable);
#define OSA_DPRINTF(pars)     { {OsaUint32 stackAvailable; OsaGetKernelStackAvailable(&stackAvailable);} printk pars;}
#define OSA_DUMP_CALLSTACK    // Debug tool for checking call stack used by OsaGetKernelStackAvailable

#ifdef __cplusplus
}
#endif

#else // not STACK_CHECK_ENABLED == 1

#define OSA_DPRINTF(pars)     printk pars
#ifdef OSA_DUMP_CALLSTACK
#undef  OSA_DUMP_CALLSTACK    // Debug tool for checking call stack used by OsaGetKernelStackAvailable    
#endif

#endif  // end of STACK_CHECK_ENABLED == 1


#define OSA_EPRINTF(pars)     { printk("%s(%d) : ", __FILE__, __LINE__); printk pars; }
#define OSA_BREAKPOINT()
#define OSA_ASSERT(condition) { if (!(condition)) { printk("%s(%d) : ASSERT(%s) FAILED\n", __FILE__, __LINE__, #condition); } }
#define OSA_FATAL(x)          { printk("%s(%d) : FATAL ERROR\n", __FILE__, __LINE__); }

#else // no DBG

#ifdef OSA_DUMP_CALLSTACK
#undef  OSA_DUMP_CALLSTACK    // Debug tool for checking call stack used by OsaGetKernelStackAvailable    
#endif
#define OSA_DPRINTF(x)
#define OSA_EPRINTF(pars) { printk("%s(%d) : ", __FILE__, __LINE__); printk pars; }
#define OSA_BREAKPOINT()
#define OSA_ASSERT(x)
#define OSA_FATAL(x)

#endif



#ifdef __cplusplus
extern "C" {
#else
  typedef unsigned long wchar_t;
#endif
#define new ident_new
#define delete ident_delete
#define class ident_class
#define virtual ident_virtual     //KERNEL_VERSION(2,4,0): conflict with "virtual" in header file <linux/mm.h>  
#define namespace ident_namespace
#define private ident_private


#if LINUX_VERSION_CODE < KERNEL_VERSION(2,4,0)
#include <linux/string.h>
#include <asm/system.h>
#include <asm/spinlock.h> 
#include <linux/ioctl.h> 
#include <linux/delay.h>
#include <linux/string.h>
#include <linux/pci.h> 
#include <asm/system.h>
#include <linux/kernel.h>
//#include <linux/module.h>
#include <linux/ioport.h>
#include <asm/errno.h>
#include <asm/atomic.h>
#include <asm/poll.h>
//#include <linux/proc_fs.h>  // for proc_register(), proc_unregister()
//#include <asm/semaphore.h>
#else

#ifdef DAVINCI
// Davinci build
#ifdef asmlinkage
#undef asmlinkage
#endif

#ifndef asmlinkage
#define asmlinkage  
#endif

// needed for printk
#include <linux/kernel.h>

#endif // DAVINCI

#include <asm/system.h>     // for local_irq_save, local_irq_restore
#include <asm/errno.h>      // for -EFAULT
#include <asm/atomic.h>  
#if LINUX_VERSION_CODE <= KERNEL_VERSION(2,4,2)
// rh7.1
#include <linux/fs.h>
#endif

//#include <linux/iso_fs.h>   // for leak_check_malloc() and leak_check_free_s()

#include <linux/string.h>   // added because proc_fs.h is removed
#include <asm/ioctl.h>      // added because proc_fs.h is removed
//#include <asm/semaphore.h>
#endif


#undef new
#undef delete
#undef class
#undef virtual    // KERNEL_VERSION(2,4,0)
#undef namespace  // KERNEL_VERSION(2,4,0)
#undef private

// Forward declaration
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,4,0)
struct wait_queue;
#else
#endif

struct poll_table_struct;
struct file;

// In Linux a user event exists as a struct in user space and as a device in OSA, in the driver we are just allowed
  // to refer to it using a opaque pointer
typedef void *OsaUserEventP;

typedef atomic_t OsaInterlockedInt, *OsaInterlockedIntP;

void OsaInitLinuxRtl(void);

#define OsaInterlockedRead(v)     atomic_read(v)
#define OsaInterlockedWrite(v, i) atomic_set(v, i)
#define OsaAddrOfFunc(f) (&f)

int osalinux_wait_event_timeout(OsaUint32 *pEventSet, void **ppWaitQueue, OsaUint32 nWaitMaxMs);

void *osalinux_kmalloc(size_t size);
void osalinux_kfree(void *ptr);


// Forward declaration
struct pt_regs;

int osalinux_request_irq(unsigned int irq, void *dev_id);
void OsaLinuxInterruptCallback(int irq, void *context, struct pt_regs *regs);

void osalinux_free_irq(unsigned int irq, void *dev_id);
void *osalinux_ioremap(unsigned long offset, unsigned long size);
void osalinux_iounmap(void *addr);

void osalinux_wake_up(void **wait_queue);
void osalinux_poll_wait(struct file * filp, void **wait_address, struct poll_table_struct *p);

size_t  wcslen(const wchar_t *psz);
wchar_t *wcscpy(wchar_t *strDestination, const wchar_t *strSource);
wchar_t *wcsncpy(wchar_t *strDest, const wchar_t *strSource, size_t count);
wchar_t towupper(wchar_t c);
char *wcsztosz(char *strDest, const wchar_t *strSource);
char *_strrev(char *szBuf);

OsaUint32 getbits(OsaUint32 x, int p, int n);

// inline functions
// For some reason related to C/C++ inclusion of header file I could not include the asm/io.h header
// Therefore we use macros instead for these low level functions.
#if 1 // I could not get the macro to compile without getting the 'warning: implicit declaration of function' /HJ 2000-09-14

OsaUint32   OsaReadReg32(OsaUint32 pRegister);
OsaUint16   OsaReadReg16(OsaUint32 pRegister);
OsaUint8    OsaReadReg8 (OsaUint32 pRegister);

void        OsaWriteReg32(OsaUint32 pRegister, OsaUint32 nValue);
void        OsaWriteReg16(OsaUint32 pRegister, OsaUint16 nValue);
void        OsaWriteReg8 (OsaUint32 pRegister, OsaUint8  nValue);

// Read and write many bytes from the same memory register
void        OsaReadRegBuffer8  (OsaUint32 pRegister, OsaUint8  *pBuffer, OsaUint32 coCount);
void        OsaWriteRegBuffer8 (OsaUint32 pRegister, OsaUint8  *pBuffer, OsaUint32 coCount);

OsaUint32   OsaReadPort32(OsaUint32 pPort);
OsaUint16   OsaReadPort16(OsaUint32 pPort);
OsaUint8    OsaReadPort8 (OsaUint32 pPort);

void        OsaWritePort32(OsaUint32 pPort, OsaUint32 nValue);
void        OsaWritePort16(OsaUint32 pPort, OsaUint16 nValue);
void        OsaWritePort8 (OsaUint32 pPort, OsaUint8  nValue);

// Read and write many bytes from the same I/O port
void        OsaReadPortBuffer8 (OsaUint32 pPort, OsaUint8  *pBuffer, OsaUint32 coCount);
void        OsaWritePortBuffer8(OsaUint32 pPort, OsaUint8  *pBuffer, OsaUint32 coCount);

#else
#define OsaReadReg32(pRegister) readl(pRegister)
#define OsaReadReg16(pRegister) readw(pRegister)
#define OsaReadReg8 (pRegister) readb(pRegister)

#define OsaWriteReg32(pRegister, nValue) writel(nValue, pRegister)
#define OsaWriteReg16(pRegister, nValue) writew(nValue, pRegister)
#define OsaWriteReg8 (pRegister, nValue) writeb(nValue, pRegister)

// Read and write many bytes from the same memory register
#define OsaReadRegBuffer8  (pRegister, pBuffer, coCount)         /*
*/ {                                                             /*
*/  OsaUint32 coReads;                                           /*
*/  for (coReads = 0; coReads < coCount; coReads++, pBuffer++) { /*
*/    *pBuffer = readb(pRegister);                               /* 
*/  }                                                            /*
*/ }


#define  OsaWriteRegBuffer8 (pRegister, pBuffer, coCount)            /*
*/ {                                                                 /*
*/   OsaUint32 coWrites;                                             /*
*/   for (coWrites = 0; coWrites < coCount; coWrites++, pBuffer++) { /*
*/     writeb(*pBuffer, pRegister);                                  /*
*/   }                                                               /*
*/ }

#define OsaReadPort32(pPort) inl(pPort)
#define OsaReadPort16(pPort) inw(pPort)
#define OsaReadPort8 (pPort) inb(pPort)

#define OsaWritePort32(pPort, nValue) outl(nValue, pPort)
#define OsaWritePort16(pPort, nValue) outw(nValue, pPort)
#define OsaWritePort8 (pPort, nValue) outb(nValue, pPort)

// Read and write many bytes from the same I/O port
#define OsaReadPortBuffer8 (pPort, pBuffer, coCount) insb(pPort, pBuffer, coCount)
#define OsaWritePortBuffer8(pPort, pBuffer, coCount) outsb(pPort, pBuffer, coCount)
#endif

// DMA routines
void *osalinux_alloc_dma_common_buf(void *pPciDev, size_t size, unsigned int *pbus_addr);
void osalinux_free_dma_common_buf(void *pPciDev, size_t size, unsigned int log_addr, unsigned int bus_addr);

// dpc routines
void osalinux_request_deferred_board_notification(
  void (*pfnDeferredBoardNotificationCallback)(void *, void *, void *),
  void *internalDeviceRep,
  void *pIrp,
  void *pContext
);

OsaBool32 osalinux_request_delayed_dpc(
  void (*pfnDpc)(void *, void *, void *, void *),
  void *pDpcContext,
  void *pContext,
  void *pArg1,
  void *pArg2,
  unsigned long dueTimeInMicroSeconds
);

void osalinux_request_dpc(
  void (*pfnDpc)(void *, void *, void *, void *),
  void *pDpcContext,
  void *pContext,
  void *pArg1,
  void *pArg2
);

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)
void osalinux_save_flags_cli(unsigned long *flags);
void osalinux_restore_flags(unsigned long flags);
#endif

#ifdef __cplusplus
}
#endif

#endif
